function Edot=HColaneri(t,Evec,lpvsys,ptrajectory,g,c,fmp,msk)
%Hamiltonian system based on COLANERI's paper. 

% Construct parameter vector at time t from basis functions
p=ptrajectory(t,c,fmp,msk);

% Evaluate state-space matrices of LPV system at current parameter vector
[A,B,C,D]=lpvsys(p);
[nz,nw] = size(D);
nx = size(A,1);
r=2*nx; c=fix(length(Evec)/r);

% Convert E from vector to matrix form
E = reshape(Evec,[r c]);

% Compute Edot
% (Use Eqn 10 in 2009 Automatica paper by Cantoni and Sandberg)
R=g^2*eye(nw)-D'*D;
S=g^2*eye(nz)-D*D';
H11=A+B*(R\D')*C;
H22=-H11';
H21=-g^2*C'/S*C;
H12=B/R*B';
Edot=[H11 H12;H21 H22]*E;

% Vectorize derivative of E
Edot = Edot(:);
